==> smtpfeed_1.19.orig.tar.gz: smtpfeed-1.19/log.c <==
/*
 * Copyright (c) 1997-2003 Motonori Nakamura <motonori@media.kyoto-u.ac.jp>
 * Copyright (c) 1997-2003 WIDE Project
 * Copyright (c) 1997-2003 Kyoto University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by WIDE Project and
 *      its contributors.
 * 4. Neither the name of the Project, the University nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static char *_id_ = "$Id: log.c,v 1.12 2003/08/20 05:12:58 motonori Exp $";
#endif

# include "common.h"
# include "extern.h"

#define	SYSLOG_FD
#ifdef	SYSLOG_FD
static int syslog_fd = -1;
#endif

void
init_log()
{
#ifdef	SYSLOG_FD
	int fd_before, fd_after;
#endif

	if (cnf.log_facility < 0)
		return;

#ifdef	SYSLOG_FD
	/* a hack to get the file descriptor for /dev/log */
	fd_before = open ("/dev/null", O_WRONLY, 0);
	if (fd_before >= 0)
		close(fd_before);
#endif

	openlog("smtpfeed", LOG_NDELAY|LOG_PID, cnf.log_facility);

#ifdef	SYSLOG_FD
	fd_after = open ("/dev/null", O_WRONLY, 0);
	if (fd_after >= 0)
		close(fd_after);
	if (fd_before >= 0 && fd_before != fd_after)
	{
		syslog_fd = fd_before;
		if (cnf.debug & DEBUG_TRANS)
		log(LOG_DEBUG, "file descriptor for syslog: %d", syslog_fd);
	}
#endif
}

#ifdef	SYSLOG_FD
#define	SLEEPTIME	100000	/* micro second */
#define	SYSLOG_RETRY	1000
#endif

void
# ifdef __STDC__
log(int level, const char *fmt, ...)
# else
log(level, fmt, va_alist)
int level;
char *fmt;
va_dcl
# endif
{
	char buf[MAXLINE];
	int len, save_errno;
	VA_LOCAL_DECL

	if (cnf.log_facility == LOG_NONE)
		return;
	buf[0] = '\0';
	save_errno = errno;
	if (env.queueid != NULL)
		strncpy(buf, env.queueid, MAXLINE);
	else if (env.envid != NULL)
		strncpy(buf, env.envid, MAXLINE);
	else
		strcpy(buf, "NOID");
	len = strlen(buf);
	if (len >= MAXLINE-2)
		len = 0;
	else {
		strcat(buf, ": ");
		len += 2;
	}

	VA_START(fmt);
	vsnprintf(buf+len, sizeof(buf)-len, fmt, ap);
	VA_END;

	if (cnf.log_facility == LOG_STDERR) {
		fprintf(stderr, "%s\n", buf);
	} else {
#ifdef	SYSLOG_FD
		int retry = SYSLOG_RETRY;

		errno = 0;
#endif
		syslog(level, "%s", buf);
#ifdef	SYSLOG_FD
		while (errno == ENOBUFS && syslog_fd >= 0)
		{
			/* retry to write out into syslog */
			struct timeval timeout;
			static fd_set slmask;
			int r;

			timeout.tv_sec = 0;
			timeout.tv_usec = SLEEPTIME;
			FD_ZERO(&slmask);
			FD_SET(syslog_fd, &slmask);
			r = select(syslog_fd + 1, (fd_set *)NULL, &slmask,
				   (fd_set *)NULL, &timeout);
			if (r > 0)
			{
				/* ready to write */
				errno = 0;
				syslog(level, "%s", buf);
			}
			if (r < 0 || --retry <= 0)
				break;
		}
#endif
	}
	errno = save_errno;
}

==> smtpfeed-1.23.tar.gz: smtpfeed-1.23/log.c <==
/*
 * Copyright (c) 1997-2012 Motonori Nakamura <motonori@wide.ad.jp>
 * Copyright (c) 1997-2012 WIDE Project
 * Copyright (c) 1997-2003 Kyoto University
 * Copyright (c) 2012 National Institute of Informatics
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by WIDE Project and
 *      its contributors.
 * 4. Neither the name of the Project, the University nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static char *_id_ = "$Id: log.c,v 1.14 2012/06/07 07:51:35 motonori Exp $";
#endif

# include "common.h"
# include "extern.h"

#define	SYSLOG_FD
#ifdef	SYSLOG_FD
static int syslog_fd = -1;
#endif

void
init_logg()
{
#ifdef	SYSLOG_FD
	int fd_before, fd_after;
#endif

	if (cnf.log_facility < 0)
		return;

#ifdef	SYSLOG_FD
	/* a hack to get the file descriptor for /dev/log */
	fd_before = open ("/dev/null", O_WRONLY, 0);
	if (fd_before >= 0)
		close(fd_before);
#endif

	openlog("smtpfeed", LOG_NDELAY|LOG_PID, cnf.log_facility);

#ifdef	SYSLOG_FD
	fd_after = open ("/dev/null", O_WRONLY, 0);
	if (fd_after >= 0)
		close(fd_after);
	if (fd_before >= 0 && fd_before != fd_after)
	{
		syslog_fd = fd_before;
		if (cnf.debug & DEBUG_TRANS)
		logg(LOG_DEBUG, "file descriptor for syslog: %d", syslog_fd);
	}
#endif
}

#ifdef	SYSLOG_FD
#define	SLEEPTIME	100000	/* micro second */
#define	SYSLOG_RETRY	1000
#endif

void
# ifdef __STDC__
logg(int level, const char *fmt, ...)
# else
logg(level, fmt, va_alist)
int level;
char *fmt;
va_dcl
# endif
{
	char buf[MAXLINE];
	int len, save_errno;
	VA_LOCAL_DECL

	if (cnf.log_facility == LOG_NONE)
		return;
	buf[0] = '\0';
	save_errno = errno;
	if (env.queueid != NULL)
		strncpy(buf, env.queueid, MAXLINE);
	else if (env.envid != NULL)
		strncpy(buf, env.envid, MAXLINE);
	else
		strcpy(buf, "NOID");
	len = strlen(buf);
	if (len >= MAXLINE-2)
		len = 0;
	else {
		strcat(buf, ": ");
		len += 2;
	}

	VA_START(fmt);
	vsnprintf(buf+len, sizeof(buf)-len, fmt, ap);
	VA_END;

	if (cnf.log_facility == LOG_STDERR) {
		fprintf(stderr, "%s\n", buf);
	} else {
#ifdef	SYSLOG_FD
		int retry = SYSLOG_RETRY;

		errno = 0;
#endif
		syslog(level, "%s", buf);
#ifdef	SYSLOG_FD
		while (errno == ENOBUFS && syslog_fd >= 0)
		{
			/* retry to write out into syslog */
			struct timeval timeout;
			static fd_set slmask;
			int r;

			timeout.tv_sec = 0;
			timeout.tv_usec = SLEEPTIME;
			FD_ZERO(&slmask);
			FD_SET(syslog_fd, &slmask);
			r = select(syslog_fd + 1, (fd_set *)NULL, &slmask,
				   (fd_set *)NULL, &timeout);
			if (r > 0)
			{
				/* ready to write */
				errno = 0;
				syslog(level, "%s", buf);
			}
			if (r < 0 || --retry <= 0)
				break;
		}
#endif
	}
	errno = save_errno;
}
